searchData={"items":[{"type":"module","title":"Shoehorn","doc":"Shoehorn helps you handle OTP application failures\n\nShoehorn needs to be enabled when building your project. See the Shoehorn\nREADME.md for usage.","ref":"Shoehorn.html"},{"type":"module","title":"Shoehorn.DefaultHandler","doc":"Default handler that ignores all events","ref":"Shoehorn.DefaultHandler.html"},{"type":"behaviour","title":"Shoehorn.Handler","doc":"A behaviour module for implementing handling of failing applications\n\nA Shoehorn.Handler is a module that knows how to respond to specific\napplications going down. There are two types of failing applications.\nThe first is an application that fails to initialize and the second\nis an application that stops while it is running.","ref":"Shoehorn.Handler.html"},{"type":"behaviour","title":"Example - Shoehorn.Handler","doc":"The Shoehorn.Handler behaviour requires developers to implement two\ncallbacks.\n\nThe `init` callback sets up any state the handler needs.\n\nThe `application_started` callback is called when an application\nstarts up.\n\nThe `application_exited` callback processes the incoming failure and replies\nwith the reaction that `Shoehorn` should take in case of application failure.\n\n        defmodule Example.ShoehornHandler do\n          @behaviour Shoehorn.Handler\n\n          @impl Shoehorn.Handler\n          def init(_opts) do\n            {:ok, %{restart_counts: 0}}\n          end\n\n          @impl Shoehorn.Handler\n          def application_started(app, state) do\n            {:continue, state}\n          end\n\n          @impl Shoehorn.Handler\n          def application_exited(:non_essential_app, _reason,  state) do\n            {:continue, state}\n          end\n\n          def application_exited(:essential_app, _reason, %{restart_counts: restart_counts} = state) when restart_counts < 2 do\n            # repair actions to make before restarting\n            Application.ensure_all_started(:essential_app)\n            {:continue, %{state | restart_counts: restart_counts + 1}}\n          end\n\n          def application_exited(_, state) do\n            {:halt, state}\n          end\n        end\n\nWe initialize our `Shoehorn.Handler` with a restart count for state\nby calling `init` with the configuration options from our shoehorn\nconfig.\n\nWhen we have an application startup, we will put a message on the\nconsole to notify the developer.\n\nWhen we have a non-essential application fail we return `:continue` to\ninform the system to keep going like nothing happened.\n\nWe restart the essential application of our system two times, and\nthen we tell the system to halt if starting over wasn't fixing the\nsystem.","ref":"Shoehorn.Handler.html#module-example"},{"type":"callback","title":"Shoehorn.Handler.application_exited/3","doc":"Callback for handling application crashes\n\nCalled with the application name, cause, and the handler's\nstate. It must return a tuple containing the `reaction` that the\n`Shoehorn` should take, and the new state\nof the handler.\n\nThe code that you execute here can be used to notify or capture some\ninformation before halting the system. This information can later\nbe used to recreate the issue or debug the problem  causing the\napplication to exit.\n\nUse `application_exited` as a place for a last-ditch effort to fix the\nissue and restart the application.  Ideally, capture\nsome information on the system state, and solve it upstream. Shoehorn\nrestarts should be used as a splint to keep a critical system\nrunning.\n\nThe default implementation returns the previous state, and a `:halt`\nreaction.","ref":"Shoehorn.Handler.html#c:application_exited/3"},{"type":"callback","title":"Callback Timeout - Shoehorn.Handler.application_exited/3","doc":"This callback is limited in the amount of time it has to execute by\na shutdown timer. If the callback does not return within the defined\nshutdown time, the node is instructed to halt. The default shutdown\ntime is 30 seconds but this value can be changed in the application\nconfig.","ref":"Shoehorn.Handler.html#c:application_exited/3-callback-timeout"},{"type":"callback","title":"Shoehorn.Handler.application_started/2","doc":"Callback for handling application starts\n\nCalled with the application name, and the handler's state. It must return a\ntuple containing the `reaction` that the `Shoehorn` should take, and the new\nstate of the handler.\n\n          def application_exited(:essential_app, _reason, state) do\n            # repair actions to make before restarting\n            # notify someone of the crash and the details\n            # log debug data\n            Application.ensure_all_started(:essential_app)\n            {:continue, %{state | restart_counts: restart_counts + 1}}\n          end\n\nThe default implementation returns unchanged state, and a `:continue`\nreaction.","ref":"Shoehorn.Handler.html#c:application_started/2"},{"type":"function","title":"Shoehorn.Handler.init/1","doc":"","ref":"Shoehorn.Handler.html#init/1"},{"type":"callback","title":"Shoehorn.Handler.init/1","doc":"Callback to initialize the handle\n\nThe callback will receive a keyword list of all the configuration\nfor shoehorn.\n\nThe callback must return a tuple of `{:ok, state}`. Where state is\nthe initial state of the handler. The system will halt if the\nreturn is anything other than `:ok`.","ref":"Shoehorn.Handler.html#c:init/1"},{"type":"function","title":"Shoehorn.Handler.invoke/3","doc":"","ref":"Shoehorn.Handler.html#invoke/3"},{"type":"function","title":"Shoehorn.Handler.invoke/4","doc":"","ref":"Shoehorn.Handler.html#invoke/4"},{"type":"type","title":"Shoehorn.Handler.cause/0","doc":"The cause that is firing the handler","ref":"Shoehorn.Handler.html#t:cause/0"},{"type":"type","title":"Shoehorn.Handler.opts/0","doc":"","ref":"Shoehorn.Handler.html#t:opts/0"},{"type":"type","title":"Shoehorn.Handler.reaction/0","doc":"The reaction letting `Shoehorn` know what to do\n\n* `:continue` - keep the system going like nothing happened\n* `:halt`    - stop the application and bring the system down","ref":"Shoehorn.Handler.html#t:reaction/0"},{"type":"type","title":"Shoehorn.Handler.t/0","doc":"","ref":"Shoehorn.Handler.html#t:t/0"},{"type":"exception","title":"Shoehorn.ReleaseError","doc":"Error type for release boot script errors","ref":"Shoehorn.ReleaseError.html"},{"type":"extras","title":"Shoehorn","doc":"# Shoehorn\n\n[![CircleCI](https://circleci.com/gh/nerves-project/shoehorn.svg?style=svg)](https://circleci.com/gh/nerves-project/shoehorn)\n[![Hex version](https://img.shields.io/hexpm/v/shoehorn.svg \"Hex version\")](https://hex.pm/packages/shoehorn)\n\nShoehorn helps you handle OTP application failures","ref":"readme.html"},{"type":"extras","title":"Motivation - Shoehorn","doc":"By default, the Erlang VM exits when OTP applications unexpectedly stop. This\ncan happen if an application's `Application.start/2` callback crashes or if a\n`GenServer` crashes repeatedly and takes down the application's supervision\ntree. Either way, recovery needs to happen outside of the Erlang VM.\n\nShoehorn provides a way of handling this inside the Erlang VM to allow you to\ndebug, restart an application, switch to a recovery mode, or something else of\nyour choosing. It does this by creating a custom release start script\n(`shoehorn.boot`) and exposing the `Shoehorn.Handler` behaviour for your code to\ndecide what to do. The custom release start script turns off the default OTP\napplication mode that exits the VM on unexpected errors and orders application\nstarts to make sure that the handler is available.\n\nShoehorn has another benefit of letting you influence the OTP application start\norder. Dependencies still determine the overall ordering, but it's possible to\nsort applications earlier via Shoehorn's `:init` option. This can let you\nimprove the apparent release startup time on slow platforms.","ref":"readme.html#motivation"},{"type":"extras","title":"Usage - Shoehorn","doc":"Run `mix release.init` on your project and then add `shoehorn` to your `mix\nreleases` configuration in the `mix.exs` (replace `:simple_app`):\n\n```elixir\n  def project do\n    [\n      ...\n      releases: releases()\n    ]\n  end\n\n  def releases do\n    [\n      simple_app: [\n        steps: [&Shoehorn.Release.init/1, :assemble]\n      ]\n    ]\n  end\n\n  defp deps do\n    [\n      {:shoehorn, \"~> 0.9.2\"}\n    ]\n  end\nend\n```\n\nCreate a release:\n\n```sh\nmix release\n```\n\nNext, take a look at the start script so that you can see how your application\nwill now be started and how it compares to the default `startup.script`. Open\n`_build/dev/rel/simple_app/releases/0.1.0/shoehorn.script` and go to the end.\nYou should see something like the following:\n\n```erlang\n     {progress,applications_loaded},\n     {apply,{application,start_boot,[kernel,permanent]}},\n     {apply,{application,start_boot,[stdlib,permanent]}},\n     {apply,{application,start_boot,[compiler,permanent]}},\n     {apply,{application,start_boot,[elixir,permanent]}},\n     {apply,{application,start_boot,[logger,permanent]}},\n     {apply,{application,start_boot,[crypto,permanent]}},\n     {apply,{application,start_boot,[shoehorn,permanent]}},\n     {apply,{application,start_boot,[sasl,permanent]}},\n     {apply,{application,start_boot,[simple_app,temporary]}},\n     {progress,started}\n```\n\nThis shows the order that applications will be started and their mode.\nApplications marked `permanent` will exit the VM if they stop expectantly.\nShoehorn will change as much as it can to `temporary` so that it (and by\nextension, you) can control what happens.\n\nTo start your release using the `shoehorn` boot script, run:\n\n```sh\nRELEASE_BOOT_SCRIPT=shoehorn _build/dev/rel/simple_app/bin/simple_app start_iex\n```\n\nIt should work as expected with the possible exception that the Erlang VM won't\nexit for any of the OTP applications marked `temporary`.\n\nNow let's configure `shoehorn` to do something more interesting by adding some\nminimal configuration. This is hypothetical unless you're using Nerves:\n\n```elixir\n# config/config.exs\n\nconfig :shoehorn,\n  init: [:nerves_runtime, :nerves_pack]\n```\n\nShoehorn will generate a release script that starts `:nerves_runtime` and its\ndependencies as soon as it can. Then it will start `:nerves_pack` and its\ndependencies. Then it will start the remainder of the applications in the\nproject. Inspect the `shoehorn.script` file in the release directory to verify\nthis.\n\nUse the `init` application list to prioritize OTP applications that are needed\nfor early on or for error recovery. In the example above, we initialize the\nruntime, bring up the network (in `:nerves_pack`), and ensure that we can\nreceive new firmware updates. Now, if `simple_app` fails to start, the device\nwould still be in a state where it can receive new firmware over the network.","ref":"readme.html#usage"},{"type":"extras","title":"Handling application failures - Shoehorn","doc":"The Erlang VM will respond to application failures differently, depending on the\n_mode_ specified when the application started. The modes are:\n\n* `:permanent` - if the application terminates, all other applications and the\n  entire node are also terminated.\n* `:transient` - if the application terminates with `:normal reason`, it is\n  reported but no other applications are terminated. However, if the application\n  terminates abnormally, all other applications and the entire node are also\n  terminated.\n* `:temporary` - if the application terminates, it is reported but no other\n  applications are terminated (the default behaviour).\n\nUnless overridden in the Mix release using the [`:applications`\noption](https://hexdocs.pm/mix/Mix.Tasks.Release.html#module-options), Shoehorn\nmost applications as `:temporary` and monitors application events by registering\nwith the Erlang [error_logger](http://erlang.org/doc/man/error_logger.html).\n\nApplication start and exit events will attempt to execute a callback to the\nconfigured `Shoehorn.Handler` module. By default, the module\n`Shoehorn.DefaultHandler` will be called. This module is configured to continue\nthe Erlang VM if any OTP application were to exit, for any reason. In\nproduction, you may want to customize the action on failure so you can gather\nforensics or perform updates to the node.  You can do this by overriding the\nhandler in the prod env of your application config.\n\n```elixir\n# config/prod.exs\n\nconfig :shoehorn,\n  handler: SimpleApp.ShoehornHandler\n```\n\nMore advanced failure cases can be handled by providing your own module that\nimplements the `Shoehorn.Handler` behaviour. For example, the Erlang `:ssh`\napplication used to exit when subjected to a brute force attack (this seems like\nit has been fixed). Instead of the default production behaviour of forcing the\nnode to restart, we can restart the application.\n\n```elixir\ndefmodule Example.RestartHandler do\n  @behavior Shoehorn.Handler\n\n  def init(_opts) do\n    {:ok, :no_state}\n  end\n\n  def application_started(_app, state) do\n    {:continue, state}\n  end\n\n  def application_exited(:ssh, _reason, state) do\n    Logger.error(\"Stop bothering ssh!\")\n    Process.sleep(1000)\n    Application.ensure_all_started(:ssh)\n    {:continue, state}\n  end\n\n  def application_exited(app, _reason, state) do\n    Logger.error(\"Application stopped! #{inspect(app)} #{inspect(state)}\")\n    {:halt, state}\n  end\nend\n```\n\nThe `application_exited/3` callback is limited in the amount of time is has to\nexecute by setting a shutdown timer. If the callback does not return within the\ndefined shutdown time, the node is instructed to halt. The default shutdown time\nis 30 seconds but this value can be changed in the application config:\n\n```elixir\n# config/config.exs\n\nconfig :shoehorn,\n  shutdown_timer: 50_000 # 50 Seconds\n```\n\nHave a look at the [example\napplication](https://github.com/nerves-project/shoehorn/tree/main/example) for\nmore info on implementing custom strategies.","ref":"readme.html#handling-application-failures"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.9.2 - 2024-03-04 - Changelog","doc":"* Updates\n  * Improve error message when an OTP application isn't found when building the\n    OTP release script. It's usually due to a dependency typo or wrong targets\n    spec, so point to that.","ref":"changelog.html#v0-9-2-2024-03-04"},{"type":"extras","title":"v0.9.1 - 2022-04-04 - Changelog","doc":"* Updates\n  * Improve detection of invalid applications being passed in the `:init` and\n    `:last` options.\n  * Fall back to a reasonable default when trying to get application modes from\n    the release options. This fixes an exception when building the release.\n  * Support release configuration via the release options in a project's\n    `mix.exs`. Add a `:shoehorn` key to the release parameters to set `:init`,\n    `:last` or the `:exxtra_dependencies` options.","ref":"changelog.html#v0-9-1-2022-04-04"},{"type":"extras","title":"v0.9.0 - 2022-04-03 - Changelog","doc":"This is a major update to Shoehorn that includes **breaking changes**:\n\n* The `:init` configuration option only supports applications now. MFAs are no\n  longer supported and moved to `runtime.exs` or an `Application.start`\n  callback.\n* References to `use Shoehorn.Handler` need to be updated to `@behaviour\n  Shoehorn.Handler`. This may require implementing additional functions.\n* Elixir 1.9 is no longer supported. Please update to Elixir 1.10 or later.\n\nThe main update to Shoehorn is to move all application startup to the boot\nscript. This noticeably improves boot time on many Nerves platforms due to boot\nscripts being able to load files without traversing the entire Erlang module\npath list. These traversals are amazingly slow (sometimes seconds) due to a\ncombination of SquashFS slowness in this area and slow overall IO.\n\nUsing boot scripts to load all applications has some important improvements in\naddition to performance:\n\n* Application start order is deterministic and computed at compile-time. If you\n  want to see the order, take a look at the end of the `shoehorn.script` in your\n  release directory.\n* Shoehorn alphabetizes the start of applications that could be ordered\n  arbitrarily. This minimizes changes in start ordering when dependencies are\n  added or removed.\n* It enables experimental features like providing additional dependencies (using\n  the `:extra_dependencies` configuration key) or hinting that dependencies get\n  started as late as possible (the `:last` configuration key)\n* You can remove the `:app` configuration key from your Shoehorn configuration.\n  It's no longer needed.\n\nAside from the change from a macro to a behaviour and possibly needing to\nimplement callback functions, `Shoehorn.Handler` implementations work the same\nas before.","ref":"changelog.html#v0-9-0-2022-04-03"},{"type":"extras","title":"v0.8.0 - 2021-10-31 - Changelog","doc":"Shoehorn v0.8.0 completely removes support for Distillery.","ref":"changelog.html#v0-8-0-2021-10-31"},{"type":"extras","title":"v0.7.0 - Changelog","doc":"Shoehorn 0.7.0 removes support for creating boot scripts using Distillery and\nonly supports using Elixir releases. As a result, the minimum supported version\nof Elixir is now version 1.9.","ref":"changelog.html#v0-7-0"},{"type":"extras","title":"v0.6.0 - Changelog","doc":"* Enhancements\n  * Added support for Elixir 1.9+ releases.\n  * Distillery is now an optional dependency and ~> 2.1.\n  * Updated supervisor specs and cleaned up warnings.","ref":"changelog.html#v0-6-0"},{"type":"extras","title":"v0.5.0 - Changelog","doc":"* Enhancements\n  * Exclude distillery, artificery, and mix from the release by default.\n  * Removed RPC and application overlay modules.\n  * Updated deps and docs.","ref":"changelog.html#v0-5-0"},{"type":"extras","title":"v0.4.0 - Changelog","doc":"* Enhancements\n  * Support for distillery ~> 2.0\n  * Support for Elixir ~> 1.7","ref":"changelog.html#v0-4-0"},{"type":"extras","title":"v0.3.1 - Changelog","doc":"For Shoehorn, these are our goals:\n1: Fix current issue with prod devices turning to zombies\n2: Make it really really difficult to enter a brick state ever.\n\nIt was becoming apparent that it is difficult to address goal #1 by changing the defaults without impacting goal #2 at all. We believe that its best to solve goal #1  by opting in and not by modifying the defaults.\n\nIn this release, existing projects that do not declare a handler in the\nconfig will use `Shoehorn.Handler.Ignore`.","ref":"changelog.html#v0-3-1"},{"type":"extras","title":"v0.3.0 - Changelog","doc":"The default strategy for how Shoehorn handles OTP application exits has changed.\nBefore this release, if an application were to exit the node would remain running\nand that applications would remain stopped. This may be desireable for development\nand test but is typically undesirable in production. This behaviour can be\ncustomized by configuring the `handler` in the config. For example, in dev you can\nuse the module `Shoehorn.Handler.Ignore` to prevent the node from halting on failure.\n\n  ```elixir\n  # config/dev.exs\n\n  config :shoehorn,\n    handler: Shoehorn.Handler.Ignore\n  ```\n\nCheck out the [example application](https://github.com/nerves-project/shoehorn/tree/main/example) for information on implementing custom strategies.","ref":"changelog.html#v0-3-0"},{"type":"extras","title":"v0.2.0 - Changelog","doc":"Renamed project Shoehorn.\n  It became hard to discuss this project with the name Bootloader.\n\n  * Enhancements\n    * `:init` list can contain `:application`, `{m, f, a}`, or `{Module, [args]}`.","ref":"changelog.html#v0-2-0"},{"type":"extras","title":"v0.1.3 - Changelog","doc":"* Bug Fixes\n    * Add explicit functions for each of the Distillery Plugin behaviour callbacks.","ref":"changelog.html#v0-1-3"},{"type":"extras","title":"v0.1.2 - Changelog","doc":"* Bug Fixes\n    * Only look in `:code.lib_dir()` for the Application lib dir instead of involving `mix`\n  * Enhancements\n    * Warn when an app listed in `:init` or `:app` does not exist.\n    * Output message about booting using shoehorn during `mix release`","ref":"changelog.html#v0-1-2"},{"type":"extras","title":"v0.1.1 - Changelog","doc":"* Bug Fixes\n    * Fixed issue with release path being constructed incorrectly.","ref":"changelog.html#v0-1-1"},{"type":"extras","title":"v0.1.0 - Changelog","doc":"Initial release to hex.","ref":"changelog.html#v0-1-0"}],"content_type":"text/markdown"}